@@ -35,6 +35,12 @@ except ImportError:
     pass
 
 
+if sys.version_info >= (3,):
+    string_types = str,
+else:
+    string_types = basestring,
+
+
 # macroscopic signature strings checker (we do not look inside params, `signature` will do it for us)
 FUNC_DEF = re.compile('(?s)^\\s*(?P<funcname>[_\\w][_\\w\\d]*)?\\s*'
                       '\\(\\s*(?P<params>.*?)\\s*\\)\\s*'
@@ -44,6 +50,7 @@ FUNC_DEF = re.compile('(?s)^\\s*(?P<funcname>[_\\w][_\\w\\d]*)?\\s*'
 def create_wrapper(wrapped,
                    wrapper,
                    new_sig=None,               # type: Union[str, Signature]
+                   remove_args=None,           # type: Union[str, Iterable[str]]
                    func_name=None,             # type: str
                    inject_as_first_arg=False,  # type: bool
                    add_source=True,             # type: bool
@@ -55,12 +62,13 @@ def create_wrapper(wrapped,
                    ):
     """
     Creates a signature-preserving wrapper function.
+    `create_wrapper(wrapped, wrapper, **kwargs)` is equivalent to `wraps(wrapped, **kwargs)(wrapper)`.
 
     See `@makefun.wraps`
     """
-    func_name, func_sig, doc, qualname, module_name, all_attrs = _get_args_for_wrapping(wrapped, new_sig, func_name,
-                                                                                        doc, qualname, module_name,
-                                                                                        attrs)
+    func_name, func_sig, doc, qualname, module_name, all_attrs = _get_args_for_wrapping(wrapped, new_sig, remove_args,
+                                                                                        func_name, doc, qualname,
+                                                                                        module_name, attrs)
 
     return create_function(func_sig, wrapper,
                            func_name=func_name,
@@ -93,22 +101,23 @@ def create_function(func_signature,             # type: Union[str, Signature]
                     module_name=None,           # type: str
                     **attrs):
     """
-    Creates a function with signature <func_signature> that will call <func_impl> with its arguments when called.
-    Arguments are passed as keyword-arguments when it is possible (so all the time, except for var-positional or
-    positional-only arguments that get passed as *args. Note that pos-only does not yet exist in python but this case
-    is already covered because it is supported by `Signature` objects).
+    Creates a function with signature `func_signature` that will call `func_impl` when called. All arguments received
+    by the generated function will be propagated as keyword-arguments to `func_impl` when it is possible (so all the
+    time, except for var-positional or positional-only arguments that get passed as *args. Note that positional-only
+    does not yet exist in python but this case is already covered because it is supported by `Signature` objects).
 
-    `func_signature` can be provided:
+    `func_signature` can be provided in different formats:
 
      - as a string containing the name and signature without 'def' keyword, such as `'foo(a, b: int, *args, **kwargs)'`.
-      In which case the name in the string will be used for the `__name__` and `__qualname__` of the created function
-      by default
-     - as a `Signature` object, for example created using `signature(f)` or handcrafted. In this case the `__name__`
-      and `__qualname__` of the created function will be copied from `func_impl` by default.
+       In which case the name in the string will be used for the `__name__` and `__qualname__` of the created function
+       by default
+     - as a `Signature` object, for example created using `signature(f)` or handcrafted. Since a `Signature` object
+       does not contain any name, in this case the `__name__` and `__qualname__` of the created function will be copied
+       from `func_impl` by default.
 
     All the other metadata of the created function are defined as follows:
 
-     - default `__name__` attribute (see above) can be overriden by providing a non-None `func_name`
+     - default `__name__` attribute (see above) can be overridden by providing a non-None `func_name`
      - default `__qualname__` attribute (see above) can be overridden by providing a non-None `qualname`
      - `__annotations__` attribute is created to match the annotations in the signature.
      - `__doc__` attribute is copied from `func_impl.__doc__` except if overridden using `doc`
@@ -658,6 +667,7 @@ def _get_callerframe(offset=0):
 
 def wraps(f,
           new_sig=None,         # type: Union[str, Signature]
+          remove_args=None,     # type: Union[str, Iterable[str]]
           func_name=None,             # type: str
           inject_as_first_arg=False,  # type: bool
           add_source=True,   # type: bool
@@ -668,13 +678,23 @@ def wraps(f,
           **attrs
           ):
     """
-    Decorator to create a signature-preserving wrapper function.
+    A decorator to create a signature-preserving wrapper function.
 
-    It is similar to `functools.wraps`, but relies on a proper dynamically-generated function. Therefore as opposed to
-    `functools.wraps`, the wrapper body will not be executed if the arguments provided are not
-    compliant with the signature - instead a `TypeError` will be raised before entering the wrapper body.
+    It is similar to `functools.wraps`, but
 
-    `@wraps(f)` is equivalent to
+     - relies on a proper dynamically-generated function. Therefore as opposed to `functools.wraps`,
+
+        - the wrapper body will not be executed if the arguments provided are not compliant with the signature -
+          instead a `TypeError` will be raised before entering the wrapper body.
+        - the arguments will always be received as keywords by the wrapper, when possible. See
+          [documentation](./index.md#signature-preserving-function-wrappers) for details.
+
+     - you can modify the signature of the resulting function, by providing a new one with `new_sig` or by providing a
+       list of arguments to remove in `remove_args`. See [documentation](./index.md#editing-a-signature) for details.
+       Note that you can now also easily remove arguments from the signature in order to inject them with your own -
+       with the `remove_args` parameter, see [documentation](./index.md#to-inject-a-dynamically-baked-value).
+
+    Comparison with `@with_signature`:`@wraps(f)` is equivalent to
 
         `@with_signature(signature(f),
                          func_name=f.__name__,
@@ -689,9 +709,13 @@ def wraps(f,
     `f`, so that the created function seems to be identical (except for the signature if a non-None `new_sig` is
     provided). If `new_sig` is None, we set the additional `__wrapped__` attribute on the created function, to stay
     compliant with the `functools.wraps` convention.
-    See https://docs.python.org/3/library/functools.html#functools.wraps
+
+    All options in `with_signature` remain available for overriding.
+
+    See also [python documentation on @wraps](https://docs.python.org/3/library/functools.html#functools.wraps)
     """
-    func_name, func_sig, doc, qualname, module_name, all_attrs = _get_args_for_wrapping(f, new_sig, func_name, doc,
+    func_name, func_sig, doc, qualname, module_name, all_attrs = _get_args_for_wrapping(f, new_sig, remove_args,
+                                                                                        func_name, doc,
                                                                                         qualname, module_name, attrs)
 
     return with_signature(func_sig,
@@ -704,12 +728,13 @@ def wraps(f,
                           **all_attrs)
 
 
-def _get_args_for_wrapping(wrapped, new_sig, func_name, doc, qualname, module_name, attrs):
+def _get_args_for_wrapping(wrapped, new_sig, remove_args, func_name, doc, qualname, module_name, attrs):
     """
     Internal method used by @wraps and create_wrapper
 
     :param wrapped:
     :param new_sig:
+    :param remove_args:
     :param func_name:
     :param doc:
     :param qualname:
@@ -718,7 +743,16 @@ def _get_args_for_wrapping(wrapped, new_sig, func_name, doc, qualname, module_na
     :return:
     """
     # the desired signature
-    func_sig = signature(wrapped) if new_sig is None else new_sig
+    if new_sig is not None:
+        if remove_args is not None:
+            raise ValueError("Only one of `remove_args` or `new_sig` should be provided")
+        func_sig = new_sig
+    else:
+        func_sig = signature(wrapped)
+        if remove_args:
+            if isinstance(remove_args, string_types):
+                remove_args = (remove_args,)
+            func_sig = remove_signature_parameters(func_sig, *remove_args)
 
     # the desired metadata
     if func_name is None:
@@ -779,23 +813,25 @@ def with_signature(func_signature,             # type: Union[str, Signature]
         the medatadata (func_name, doc, module_name, attrs) of the decorated function, without generating a new
         function.
     :param inject_as_first_arg: if `True`, the created function will be injected as the first positional argument of
-        the decorated function. Default=`False`
+        the decorated function. This can be handy in case the implementation is shared between several facades and needs
+        to know from which context it was called. Default=`False`
     :param func_name: provide a non-`None` value to override the created function `__name__` and `__qualname__`. If this
-        is `None` (default), the `__name__` and `__qualname__` will default to the ones of the decorated function if
-        `func_signature` is a `Signature`, or to the name defined in `func_signature` if `func_signature` is a `str`
-        and contains a non-empty name.
+        is `None` (default), the `__name__` will default to the ones of the decorated function if `func_signature` is a
+        `Signature`, or to the name defined in `func_signature` if `func_signature` is a `str` and contains a non-empty
+        name.
     :param add_source: a boolean indicating if a '__source__' annotation should be added to the generated function
         (default: True)
     :param add_impl: a boolean indicating if a '__func_impl__' annotation should be added to the generated function
         (default: True)
     :param doc: a string representing the docstring that will be used to set the __doc__ attribute on the generated
-        function. If None (default), the doc of func_impl will be used.
+        function. If None (default), the doc of the decorated function will be used.
     :param qualname: a string representing the qualified name to be used. If None (default), the `__qualname__` will
         default to the one of `func_impl` if `func_signature` is a `Signature`, or to the name defined in
         `func_signature` if `func_signature` is a `str` and contains a non-empty name.
-    :param module_name: the name of the module to be set on the function (under __module__ ). If None (default),
-        `func_impl.__module__` will be used.
-    :param attrs: other keyword attributes that should be set on the function
+    :param module_name: the name of the module to be set on the function (under __module__ ). If None (default), the
+        `__module__` attribute of the decorated function will be used.
+    :param attrs: other keyword attributes that should be set on the function. Note that the full `__dict__` of the
+        decorated function is not automatically copied.
     """
     if func_signature is None:
         # make sure that user does not provide non-default other args
@@ -834,9 +870,10 @@ def with_signature(func_signature,             # type: Union[str, Signature]
     return replace_f
 
 
-def remove_signature_parameters(s, *param_names):
+def remove_signature_parameters(s,
+                                *param_names):
     """
-    Removes the provided parameters from the signature s (returns a new signature instance).
+    Removes the provided parameters from the signature `s` (returns a new `Signature` instance).
 
     :param s:
     :param param_names: a list of parameter names to remove
@@ -855,14 +892,14 @@ def add_signature_parameters(s,             # type: Signature
                              custom_idx=-1  # type: int
                              ):
     """
-    Adds the provided parameters to the signature `s` (returns a new signature instance).
+    Adds the provided parameters to the signature `s` (returns a new `Signature` instance).
 
     :param s: the original signature to edit
     :param first: a single element or a list of `Parameter` instances to be added at the beginning of the parameter's
         list
     :param last: a single element or a list of `Parameter` instances to be added at the end of the parameter's list
     :param custom: a single element or a list of `Parameter` instances to be added at a custom position in the list.
-        that position is determined with `custom_idx`
+        That position is determined with `custom_idx`
     :param custom_idx: the custom position to insert the `custom` parameters to.
     :return: a new signature created from the original one by adding the specified parameters.
     """
@@ -927,8 +964,15 @@ def with_partial(*preset_pos_args, **preset_kwargs):
     return apply_decorator
 
 
-def partial(f, *preset_pos_args, **preset_kwargs):
+def partial(f,                 # type: Callable
+            *preset_pos_args,
+            **preset_kwargs
+            ):
     """
+    Equivalent of `functools.partial` but relies on a dynamically-created function. As a result the function
+    looks nicer to users in terms of apparent documentation, name, etc.
+
+    See [documentation](./index.md#removing-parameters-easily) for details.
 
     :param preset_pos_args:
     :param preset_kwargs:
@@ -974,7 +1018,7 @@ def gen_partial_sig(f, preset_pos_args, preset_kwargs):
     preset_kwargs = copy(preset_kwargs)
 
     # remove the first n positional, and assign/change default values for the keyword
-    if len(orig_sig.parameters) <= len(preset_pos_args):
+    if len(orig_sig.parameters) < len(preset_pos_args):
         raise ValueError("Cannot preset %s positional args, function %s has only %s args."
                          "" % (len(preset_pos_args), getattr(f, '__name__', f), len(orig_sig.parameters)))
 
